# Databricks notebook source
# MAGIC %md-sandbox
# MAGIC <img src= "/files/tables/avatar.jpg" width="100" height="100" />
# MAGIC  
# MAGIC ```
# MAGIC 
# MAGIC Name:         03-refining-lending-club-data
# MAGIC 
# MAGIC Design Phase:
# MAGIC     Author:   John Miner
# MAGIC     Date:     12-15-2020
# MAGIC     Purpose:  Demostrate how to refine data.
# MAGIC     
# MAGIC Assumptions:
# MAGIC     None
# MAGIC     
# MAGIC Algorithm:
# MAGIC     0 - Examine files & folders
# MAGIC     1 - Using dataframes to manipulate data.
# MAGIC     2 - Using spark sql to manipulate data.   
# MAGIC     
# MAGIC ```

# COMMAND ----------



# COMMAND ----------

#
# 0 - Show data file & readme file, create sample datalake
#


# COMMAND ----------

# MAGIC %fs
# MAGIC ls /databricks-datasets/lending-club-loan-stats/

# COMMAND ----------

# MAGIC %sh
# MAGIC #
# MAGIC # Show readme file
# MAGIC #
# MAGIC 
# MAGIC cat /dbfs/databricks-datasets/lending-club-loan-stats/lendingClubData.txt

# COMMAND ----------

#
# First 5000 bytes of the file
#

dbutils.fs.head('/databricks-datasets/lending-club-loan-stats/LoanStats_2018Q2.csv', 5000)

# COMMAND ----------

#
#  Create sample data lake
#

# Remove existing root folder
try:
  dbutils.fs.rm("/rissug", recurse=TRUE)  
except:
  pass

# Create new
dbutils.fs.mkdirs("/rissug")

# Create new
dbutils.fs.mkdirs("/rissug/bronze")

# Create new
dbutils.fs.mkdirs("/rissug/silver")

# Create new
dbutils.fs.mkdirs("/rissug/gold")



# COMMAND ----------

#
#  Create specific directories & load bronze zone
#

# Remove existing
try:
  dbutils.fs.rm("/rissug/bronze/lending_club", recurse=TRUE)  
except:
  pass

# Create new
dbutils.fs.mkdirs("/rissug/bronze/lending_club")

# Remove existing
try:
  dbutils.fs.rm("/rissug/silver/lending_club", recurse=TRUE)  
except:
  pass

# Create new
dbutils.fs.mkdirs("/rissug/silver/lending_club")

# Grab data file
dbutils.fs.cp(
  "/databricks-datasets/lending-club-loan-stats/LoanStats_2018Q2.csv", 
  "/rissug/bronze/lending_club/loan_stats_2018q2.csv"
)


# COMMAND ----------

# MAGIC %sh
# MAGIC #
# MAGIC #  Show the data file
# MAGIC #
# MAGIC 
# MAGIC ls /dbfs/rissug/bronze/lending_club/

# COMMAND ----------

#
#  Read in unprocessed data
#

# Read in file
raw = spark.read.csv("/rissug/bronze/lending_club/loan_stats_2018q2.csv", header=True)

# Show the data
display(raw)

# COMMAND ----------

#
# How many records?
#

raw.count()

# COMMAND ----------

#
#  2 - Process data via dataframe cmds
#

# Include functions
from pyspark.sql.functions import *

# Choose subset of data
loans1 = raw.select(
  "loan_status", 
  "int_rate", 
  "revol_util", 
  "issue_d", 
  "earliest_cr_line", 
  "emp_length", 
  "verification_status", 
  "total_pymnt", 
  "loan_amnt", 
  "grade", 
  "annual_inc", 
  "dti", 
  "addr_state", 
  "term", 
  "home_ownership", 
  "purpose", 
  "application_type", 
  "delinq_2yrs", 
  "total_acc"
)

# Create bad loan flag
loans1 = loans1.withColumn("bad_loan", (~loans1.loan_status.isin(["Current", "Fully Paid"])).cast("string"))

# Convert to number (int rate)
loans1 = loans1.withColumn('int_rate', regexp_replace('int_rate', '%', '').cast('float')) 

# Convert to number (revolving util)
loans1 = loans1.withColumn('revol_util', regexp_replace('revol_util', '%', '').cast('float')) 

# Convert to number (issue year)
loans1 = loans1.withColumn('issue_year',  substring(loans1.issue_d, 5, 4).cast('double') ) 

# Convert to number (earliest year)
loans1 = loans1.withColumn('earliest_year', substring(loans1.earliest_cr_line, 5, 4).cast('double'))

# Calculate len in yrs
loans1 = loans1.withColumn('credit_length_in_years', (loans1.issue_year - loans1.earliest_year))

# Use regex to clean up 
loans1 = loans1.withColumn('emp_length', trim(regexp_replace(loans1.emp_length, "([ ]*+[a-zA-Z].*)|(n/a)", "") ))
loans1 = loans1.withColumn('emp_length', trim(regexp_replace(loans1.emp_length, "< 1", "0") ))
loans1 = loans1.withColumn('emp_length', trim(regexp_replace(loans1.emp_length, "10\\+", "10") ).cast('float'))

# Show the data
display(loans1)

# Write out data (force)
loans1.repartition(2).write.mode("overwrite").parquet('/rissug/silver/lending_club/process01.parquet')


# COMMAND ----------

#
# Make sure empty db exists
#

# drop existing
try:
  spark.sql('DROP DATABASE mssqltips CASCADE')
except:
  pass

# create new
try:
  spark.sql('CREATE DATABASE mssqltips')
except:
  pass



# COMMAND ----------

#
#  Create loan club table 01
#

# drop existing
try:
  df = spark.sql("DROP TABLE mssqltips.loan_club_process_01")
except:
  pass

# create new
sql_stmt = """
  CREATE TABLE mssqltips.loan_club_process_01
    USING PARQUET
    LOCATION '/rissug/silver/lending_club/process01.parquet'
"""
df = spark.sql(sql_stmt)


# COMMAND ----------

# MAGIC %sql
# MAGIC -- Show 5 records
# MAGIC select * from mssqltips.loan_club_process_01 limit 5

# COMMAND ----------

#
#  3 - Process data via spark sql
#

# create temp view
raw.createOrReplaceTempView("tmp_loan_club")


# COMMAND ----------

# MAGIC %sql
# MAGIC SHOW COLUMNS IN tmp_loan_club

# COMMAND ----------

# MAGIC %sql
# MAGIC select * from tmp_loan_club limit 10

# COMMAND ----------

# MAGIC %sql
# MAGIC select
# MAGIC   loan_status, 
# MAGIC   cast(regexp_replace(int_rate, '%', '') as float) as int_rate,
# MAGIC   cast(regexp_replace(revol_util, '%', '') as float) as revol_util,
# MAGIC   cast(substring(issue_d, 5, 4) as double) as issue_year,
# MAGIC   cast(substring(earliest_cr_line, 5, 4) as double) as earliest_year,
# MAGIC   cast(substring(issue_d, 5, 4) as double) -
# MAGIC   cast(substring(earliest_cr_line, 5, 4) as double) as credit_length_in_years,
# MAGIC   cast(regexp_replace(regexp_replace(regexp_replace(emp_length, "([ ]*+[a-zA-Z].*)|(n/a)", ""), "< 1", "0"), "10\\+", "10") as float) as emp_length,
# MAGIC   verification_status, 
# MAGIC   total_pymnt,
# MAGIC   loan_amnt, 
# MAGIC   grade, 
# MAGIC   annual_inc, 
# MAGIC   dti,
# MAGIC   addr_state,
# MAGIC   term,
# MAGIC   home_ownership, 
# MAGIC   purpose, 
# MAGIC   application_type, 
# MAGIC   delinq_2yrs, 
# MAGIC   total_acc,
# MAGIC   case
# MAGIC     when loan_status = "Current" then "false"
# MAGIC     when loan_status = "Fully Paid" then "false"
# MAGIC     else "true"
# MAGIC   end as bad_loan
# MAGIC from 
# MAGIC   tmp_loan_club

# COMMAND ----------

#
#  Process data via sql
#

# Multi line here doc
sql_stmt = """

select
  loan_status, 
  cast(regexp_replace(int_rate, '%', '') as float) as int_rate,
  cast(regexp_replace(revol_util, '%', '') as float) as revol_util,
  cast(substring(issue_d, 5, 4) as double) as issue_year,
  cast(substring(earliest_cr_line, 5, 4) as double) as earliest_year,
  cast(substring(issue_d, 5, 4) as double) -
  cast(substring(earliest_cr_line, 5, 4) as double) as credit_length_in_years,
  cast(regexp_replace(regexp_replace(regexp_replace(emp_length, "([ ]*+[a-zA-Z].*)|(n/a)", ""), "< 1", "0"), "10\\+", "10") as float) as emp_length,
  verification_status, 
  total_pymnt,
  loan_amnt, 
  grade, 
  annual_inc, 
  dti,
  addr_state,
  term,
  home_ownership, 
  purpose, 
  application_type, 
  delinq_2yrs, 
  total_acc,
  case
    when loan_status = "Current" then "false"
    when loan_status = "Fully Paid" then "false"
    else "true"
  end as bad_loan
from 
  tmp_loan_club
  
"""

# Run sql & retrieve results
loans2 = spark.sql(sql_stmt)

# Write out data
loans2.repartition(2).write.mode("overwrite").parquet('/rissug/silver/lending_club/process02.parquet')


# COMMAND ----------

#
#  Create loan club table 02
#

# delete existing
try:
  df = spark.sql("DROP TABLE mssqltips.loan_club_process_02")
except:
  pass

# create new
sql_stmt = """
  CREATE TABLE mssqltips.loan_club_process_02
    USING PARQUET
    LOCATION '/rissug/silver/lending_club/process02.parquet'
"""
df = spark.sql(sql_stmt)


# COMMAND ----------

# MAGIC %sql
# MAGIC -- Show 5 records
# MAGIC select * from nyc.loan_club_process_02 limit 5

# COMMAND ----------


